@@ -1,4 +1,4 @@
-use std::{collections::HashMap, convert::TryFrom, fmt::Display};
+use std::{borrow::Cow, collections::HashMap, convert::TryFrom, fmt::Display};
 
 use chrono::{Date, DateTime, Duration, Local, Utc};
 use math::round::floor;
@@ -37,7 +37,18 @@ pub struct Elapsed {
     Key being the sec/min/hour/day, etc. identifier.
     Value being a tuple of the `diff` value as a string in pos 0, and numeric value in pos 1.
     */
-    pub cache: HashMap<TimeFrame, (String, i64)>, // format: &'static str,
+    pub cache: HashMap<TimeFrame, TimeFrameTuple>,
+    /*
+    TODO:
+    Customising display format can be done here.
+    */
+    // format: &'static str,
+    /*
+    TODO:
+    epoch could eventually be useful for running a timer that can be started via CLI.
+    A tick of the clock would then be a trigger to calculate if a minute had elapsed.
+    */
+    // epoch: i64,
 }
 
 /* Aliasing `Elapsed` because these names might make more sense, depending on use-case. */
@@ -45,6 +56,8 @@ pub struct Elapsed {
 pub type DueDateTime = Elapsed;
 /** Alias of `Elapsed`. */
 pub type TimeBetween = Elapsed;
+/** Private `TimeFrameTuple` to avoid duplicate code. */
+type TimeFrameTuple = (Cow<'static, str>, i64);
 
 impl Elapsed {
     /** Construct a new object. */
@@ -169,8 +182,8 @@ impl Elapsed {
     }
 
     /**
-    Default behaviour currently. Discards "irrelevant" time frames, for example if date is due in
-    more than a year, we'll only store `1y 6m` as opposed to `1y 6m 2w 4d`.
+    Populate `cache` with contextually aware `TimeFrame`s. Discards "irrelevant" time frames, for
+    example if date is due in more than a year, we'll only store `1y 6m` as opposed to `1y 6m 2w 4d`.
     */
     pub fn process(&mut self, clear_cache: bool) {
         /*
@@ -246,37 +259,125 @@ impl Elapsed {
 
     /** Helper function to insert a value for a `TimeFrame` into the cache. */
     pub fn cache_insert(&mut self, k: TimeFrame, v: i64) {
-        let tup = (format!("{}{}", v, k.abbrev()), v);
-        let val = self.cache.entry(k).or_insert(tup.clone());
-        *val = tup;
+        let tup = (format!("{}{}", v, k.abbrev()).into(), v);
+        if let Some(v) = self.cache.get_mut(&k) {
+            *v = tup;
+        } else {
+            self.cache.insert(k, tup);
+        }
     }
 
-    fn years(&mut self) {
+    /** Get number of years. */
+    pub fn num_years(&self) -> i64 {
+        floor((self.duration.num_weeks() / 52) as f64, 0) as i64
+    }
+
+    /** Get years between `DateTime` and `DateTime` given for context as `elapsed` style tuple. */
+    pub fn years(&mut self) -> TimeFrameTuple {
+        Self::as_tuple(TimeFrame::Year, self.num_years())
+    }
+
+    /**
+    Get years between `DateTime` and `DateTime` given for context. Can be chained to string together
+    multiple values of your choosing.
+
+    ```rust
+    let date = self.years_and().months_and().weeks();
+    println!("{}", date);
+    let silly_date = self.years_and().seconds();
+    println!("{}", silly_date);
+    ```
+
+    Results in `1y 6m 2w` the first time, or something silly the second time..
+    */
+    pub fn years_and(&mut self) -> &mut Self {
         todo!()
     }
 
-    fn months(&mut self) {
+    /** Get number of months. */
+    pub fn num_months(&self) -> i64 {
+        floor((self.duration.num_weeks() / 4) as f64, 0) as i64
+    }
+
+    /**
+    Get months between `DateTime` and `DateTime` given for context as `elapsed` style tuple.
+    */
+    pub fn months(&mut self) -> TimeFrameTuple {
+        Self::as_tuple(TimeFrame::Month, self.num_months())
+    }
+
+    /**
+    Get weeks between `DateTime` and `DateTime` given for context as `elapsed` style tuple.
+
+    Chrono provides a method to get numeric value alone, which is exposed by `Elapsed` struct
+    `duration` field.
+    */
+    pub fn weeks(&mut self) -> TimeFrameTuple {
+        Self::as_tuple(TimeFrame::Week, self.duration.num_weeks())
+    }
+
+    /**
+    Get days between `DateTime` and `DateTime` given for context as `elapsed` style tuple.
+
+    Chrono provides a method to get numeric value alone, which is exposed by `Elapsed` struct
+    `duration` field.
+    */
+    pub fn days(&mut self) -> TimeFrameTuple {
+        Self::as_tuple(TimeFrame::Day, self.duration.num_days())
+    }
+
+    /**
+    Get hours between `DateTime` and `DateTime` given for context as `elapsed` style tuple.
+
+    Chrono provides a method to get numeric value alone, which is exposed by `Elapsed` struct
+    `duration` field.*/
+    pub fn hours(&mut self) -> TimeFrameTuple {
+        Self::as_tuple(TimeFrame::Hour, self.duration.num_hours())
+    }
+
+    /**
+    Get minutes between `DateTime` and `DateTime` given for context as `elapsed` style tuple.
+
+    Chrono provides a method to get numeric value alone, which is exposed by `Elapsed` struct
+    `duration` field.
+    */
+    pub fn minutes(&mut self) -> TimeFrameTuple {
+        Self::as_tuple(TimeFrame::Minute, self.duration.num_minutes())
+    }
+
+    /**
+    Get seconds between `DateTime` and `DateTime` given for context as `elapsed` style tuple.
+
+    Chrono provides a method to get numeric value alone, which is exposed by `Elapsed` struct
+    `duration` field.
+    */
+    pub fn seconds(&mut self) -> TimeFrameTuple {
+        Self::as_tuple(TimeFrame::Second, self.duration.num_seconds())
+    }
+
+    /** Helper fn to get an elapsed style tuple. */
+    fn as_tuple(tf: TimeFrame, val: i64) -> TimeFrameTuple {
+        (format!("{}{}", val, tf.abbrev()).into(), val)
+    }
+
+    /**
+    This fn is intended to be used similarly to chaining, like so:
+
+    ```rust
+    let date = self.seconds_and().through_til(&TimeFrame::Months);
+    println!("{}", date);
+    ```
+
+    Resulting in seconds, minutes, hours, days, weeks and months being set in `cache`, and then
+    subsequently printed as `(in) 3y 2w 4d 12hr 32min 42sec (ago)`.
+    */
+    pub fn through_til(&mut self, tf: &TimeFrame) -> &mut Self {
         todo!()
     }
 
-    fn weeks(&mut self) {
-        todo!()
-    }
-
-    fn days(&mut self) {
-        todo!()
-    }
-
-    fn hours(&mut self) {
-        todo!()
-    }
-
-    fn minutes(&mut self) {
-        todo!()
-    }
-
-    fn seconds(&mut self) {
-        todo!()
+    /** Create a clone of our `cache` containing the values at time of collection. */
+    pub fn collect(&self) -> HashMap<TimeFrame, TimeFrameTuple> {
+        self.cache.clone()
     }
 }
 
